<?php declare(strict_types=1);
/**
 * JTL Plugin
 * Copyright (C) 2011 - 2020 Andreas Petermann
 *
 * PHP version 7.2+
 *
 * @copyright css-umsetzung
 * @author Andreas Petermann <info@css-umsetzung.de>
 * @package css_permanentCart
 * @license GNU
 * @version 1.00
 *
 */

namespace Plugin\css_permanentCart;

use JTL\Backend\Notification;
use JTL\Backend\NotificationEntry;
use JTL\Events\Dispatcher;
use JTL\Plugin\Bootstrapper;
use JTL\Shop;
use JTL\Router\Route;
use JTL\Minify\MinifyService;




class Bootstrap extends Bootstrapper {

	private $cssPlugin;


	/**
	 * @inheritdoc
	 */
	public function boot(Dispatcher $dispatcher) {

		parent::boot($dispatcher);
		$plugin = $this->getPlugin();
		$dispatcher->listen('backend.notification', [$this, 'checkNotification']);

		define("HOOK_CSS_KUNDE_CLASS_HOLLOGINKUNDE", intval($plugin->getID()."001"));
		if (Shop::isFrontend() === true || (isset($_REQUEST['kPlugin']) && $plugin->getID() == (int)$_REQUEST['kPlugin']) || (isset($_REQUEST['io']) && $_REQUEST['io'] == $plugin->getPluginID())) {
			$this->cssPlugin = src\cssPlugin\cssPlugin::getInstance(
				$plugin,
				$this->getDB(),
				$this->getCache()
			);
		}
		if (Shop::isFrontend() === true) {
			$cfg = $this->cssPlugin->getConfig();
			if ($cfg->pluginActive == 'N') return;

#			$dispatcher->listen('shop.hook.' . \HOOK_LETZTERINCLUDE_INC , function () use ($plugin)
#			{
#               $this->cssPlugin->hookLetzterincludeInc ();
#			});

			$dispatcher->listen('shop.hook.' . \HOOK_SMARTY_INC , function () use ($plugin)
			{
				$this->cssPlugin->hookSmartyInc ();
			});

			$dispatcher->listen('shop.hook.' . \HOOK_SMARTY_OUTPUTFILTER, function () use ($plugin)
			{
               $this->cssPlugin->hookSmartyOutputfilter();
			});
			$dispatcher->listen('shop.hook.' . \HOOK_IO_HANDLE_REQUEST, function (array $args) use ($plugin)
			{
                $this->cssPlugin->hookIoHandleRequest($args);
			});

			/*
			$dispatcher->listen('shop.hook.' . \HOOK_CSS_JS_LIST, function (array $args) use ($plugin)
			{
                $this->cssPlugin->hookCssJsList($args);
			});
			*/

			$dispatcher->listen('shop.hook.' . \HOOK_KUNDE_CLASS_HOLLOGINKUNDE, function (array $args) use ($plugin) {
				$this->cssPlugin->hookKundeClassHolloginkunde($args);
			});

			$dispatcher->listen('shop.hook.' . \HOOK_CSS_KUNDE_CLASS_HOLLOGINKUNDE, function (array $args) use ($plugin) {
				$this->cssPlugin->hookCssKundeClassHolloginkunde($args);
			});

			$dispatcher->listen('shop.hook.' . \HOOK_BESTELLABSCHLUSS_INC_BESTELLUNGINDB_ENDE  , function (array $args) use ($plugin)
			{
				$this->cssPlugin->hookBestellabschlussIncBestellungindbEnde  ($args);
			});
			$dispatcher->listen('shop.hook.' . \HOOK_WARENKORB_PAGE , function (array $args) use ($plugin)
			{
				$this->cssPlugin->hookWarenkorbPage ($args);
			});
			$dispatcher->listen('shop.hook.' . \HOOK_WARENKORB_CLASS_FUEGEEIN, function (array $args) use ($plugin)
			{
                $this->cssPlugin->hookWarenkorbClassFuegeein ($args);
			});
			$dispatcher->listen('shop.hook.' . \HOOK_WARENKORB_LOESCHE_POSITION, function (array $args) use ($plugin)
			{
                $this->cssPlugin->hookWarenkorbLoeschePosition ($args);
			});

		} else {
			if(isset($_REQUEST['io']) && $_REQUEST['io'] == $plugin->getPluginID()) {
				$dispatcher->listen('shop.hook.' . \HOOK_IO_HANDLE_REQUEST_ADMIN, function (array $args) use ($plugin)
				{
					$this->cssPlugin->hookIoHandleAdminRequest($args);
				});
			}
		}
	}

	public function installed() {
		$plugin  = $this->getPlugin();
		$obj = new \stdClass();
		$obj->cAktiv = 'Y';
		$obj->nFehlerhaft = '0';
		$this->getDB()->update('temailvorlage','kPlugin',$plugin->getID(),$obj);

		$minify = new MinifyService();
		$minify->flushCache();

		Shop::Container()->getCache()->flushTags(array(CACHING_GROUP_PLUGIN, CACHING_GROUP_TEMPLATE,'cache_'.$plugin->getPluginID()));
		parent::installed();
	}

	public function updated($oldVersion, $newVersion) {
		$plugin  = $this->getPlugin();

		$obj = $this->getDB()->query("Show columns from css_licence_log like 'notifySubscriptionMail' ",1);
		if($obj != null && isset($obj->Field)) {
			$obj = new \stdClass();
			$obj->notifyUpdate = '';
			$obj->notifyUpdateMail = '';
			$obj->cPluginVersion = $newVersion;
			$this->getDB()->update('css_licence_log', 'cPluginID', $plugin->getPluginID(), $obj);
		}
		$obj = new \stdClass();
		$obj->cAktiv = 'Y';
		$obj->nFehlerhaft = '0';
		$this->getDB()->update('temailvorlage','kPlugin',$plugin->getID(),$obj);

		$minify = new MinifyService();
		$minify->flushCache();

		Shop::Container()->getCache()->flushTags(array(CACHING_GROUP_PLUGIN, CACHING_GROUP_TEMPLATE,'cache_'.$plugin->getPluginID()));
		\error_log('updated from ' . $oldVersion . ' to ' . $newVersion);
	}


	public function uninstalled(bool $deleteData = true) {
		$plugin  = $this->getPlugin();
		if($deleteData === true) {
			$query = "SELECT * FROM INFORMATION_SCHEMA.TABLES WHERE table_schema = '".DB_NAME."' AND table_name like '".$plugin->getPluginID()."%'";
			$obj = $this->getDB()->query($query,2);
			if($obj !== null && count($obj)) {
				foreach($obj as $oTable) {
					$this->getDB()->query('DROP TABLE IF EXISTS '.$oTable->TABLE_NAME, 3);
				}
			}
		}
		$this->getDB()->delete('css_licence_log','cPluginID',$this->getPlugin()->getPluginID());
		Shop::Container()->getCache()->flushTags(array(CACHING_GROUP_PLUGIN, CACHING_GROUP_TEMPLATE,'cache_'.$plugin->getPluginID()));

		$minify = new MinifyService();
		$minify->flushCache();

		parent::uninstalled($deleteData);
	}


	public function getPluginLink($oPlugin) {
		if (version_compare(APPLICATION_VERSION, "5.2.0-beta", ">=") === true) {
			return Shop::getAdminURL() . '/' . Route::PLUGIN . '/' . $oPlugin->kPlugin;
		} else {
			return Shop::getAdminURL() . '/' . "plugin.php?kPlugin=" . $oPlugin->kPlugin;
		}
	}


	public function addSubscriptionNotify(Notification $notification,$oUpdate) {

		$days = ceil(((int) $oUpdate->notifyTimeSubscription-time()) / 86400);
		if((int) $days <=0) return;
		$notification->add(
			NotificationEntry::TYPE_DANGER,
			$oUpdate->cName,
			((int) $days == 1) ?  "<div>Ihre Subscription f&uuml;r ".$oUpdate->cDomain."</div>l&auml;uft morgen ab." : "<div>Ihre Subscription fr ".$oUpdate->cDomain."</div> l&auml;uft in  ".$days." Tagen ab.",
			$this->getPluginLink($oUpdate),
			\md5($oUpdate->kPlugin . '_subscriptionInfo_'.$oUpdate->notifyUpdate)
		);
	}


	public function addUpdateNotify(Notification $notification,$oUpdate) {
		$notification->add(
			NotificationEntry::TYPE_WARNING,
			(string) $oUpdate->cName,
			"Update Verf&uuml;gbar Version ".$oUpdate->notifyUpdate,
			$this->getPluginLink($oUpdate),
			\md5($oUpdate->kPlugin . '_updateInfo_'.$oUpdate->notifyUpdate)
		);
	}

	public function checkNotification(Notification $notification): void {
		if(Shop::has('cssCheckNotification')) return;
		Shop::set('cssCheckNotification',true);

		$query = "
					SELECT
					    p.cName,
					    p.kPlugin,
					    li.*
					FROM
					    css_licence_log AS li
					LEFT JOIN tplugin AS p
					ON
					    MD5(p.cPluginID) = MD5(li.cPluginID)
					WHERE
					    (
					        notifyUpdate != '' OR notifyTimeSubscription > 0
					    ) AND p.nStatus = 2	
		";


		$aData = $this->getDB()->query($query,2);


		if($aData !== null && count($aData)>0) {
			$aHasNotify = array();
			foreach($aData as $oData) {
				if(!empty($oData->notifyTimeSubscription)) {
					$this->addSubscriptionNotify($notification,$oData);
				}
				if(!empty($oData->notifyUpdate) && !in_array($oData->kPlugin, $aHasNotify)) {
					$aHasNotify[] = $oData->kPlugin;
					$this->addUpdateNotify($notification,$oData);
				}
			}
		}
	}
}
